@@ -41,11 +41,14 @@ BaseTab.prototype = {
         this.visible = false;
     },
 
-    show: function() {
+    show: function(animate) {
         this.visible = true;
-        this.page.opacity = 0;
         this.page.show();
 
+        if (!animate)
+            return;
+
+        this.page.opacity = 0;
         Tweener.addTween(this.page,
                          { opacity: 255,
                            time: 0.1,
@@ -77,14 +80,16 @@ BaseTab.prototype = {
 Signals.addSignalMethods(BaseTab.prototype);
 
 
-function ViewTab(label, pageActor, a11yIcon) {
-    this._init(label, pageActor, a11yIcon);
+function ViewTab(id, label, pageActor, a11yIcon) {
+    this._init(id, label, pageActor, a11yIcon);
 }
 
 ViewTab.prototype = {
     __proto__: BaseTab.prototype,
 
-    _init: function(label, pageActor, a11yIcon) {
+    _init: function(id, label, pageActor, a11yIcon) {
+        this.id = id;
+
         let titleActor = new St.Button({ label: label,
                                          style_class: 'view-tab-title' });
         titleActor.connect('clicked', Lang.bind(this, this._activate));
@@ -383,14 +388,16 @@ ViewSelector.prototype = {
         }));
     },
 
-    addViewTab: function(title, pageActor, a11yIcon) {
-        let viewTab = new ViewTab(title, pageActor, a11yIcon);
+    addViewTab: function(id, title, pageActor, a11yIcon) {
+        let viewTab = new ViewTab(id, title, pageActor, a11yIcon);
         this._tabs.push(viewTab);
         this._tabBox.add(viewTab.title);
         this._addTab(viewTab);
     },
 
     _switchTab: function(tab) {
+        let firstSwitch = this._activeTab == null;
+
         if (this._activeTab && this._activeTab.visible) {
             if (this._activeTab == tab)
                 return;
@@ -406,11 +413,13 @@ ViewSelector.prototype = {
             }
         }
 
+        // Only fade when switching between tabs,
+        // not when setting the initially selected one.
         if (!tab.visible)
-            tab.show();
+            tab.show(!firstSwitch);
 
         // Pull a Meg Ryan:
-        if (Main.overview && Main.overview.workspaces) {
+        if (!firstSwitch && Main.overview.workspaces) {
             if (tab != this._tabs[0]) {
                 Tweener.addTween(Main.overview.workspaces.actor,
                                  { opacity: 0,
@@ -433,6 +442,14 @@ ViewSelector.prototype = {
         }
     },
 
+    switchTab: function(id) {
+        for (let i = 0; i < this._tabs.length; i++)
+            if (this._tabs[i].id == id) {
+                this._switchTab(this._tabs[i]);
+                break;
+            }
+    },
+
     _switchDefaultTab: function() {
         if (this._tabs.length > 0)
             this._switchTab(this._tabs[0]);
